// ------------------------------------------------------------------------ //
//			    Atari GameFilm Project			    //
//				  Windows 95				    //
//									    //
//			    Author: Jack Thornton			    //
//		   Property of CetaSoft / Level Five, Inc.		    //
//			     All Rights Reserved			    //
// ------------------------------------------------------------------------ //
// FILE:	ASMFADER.H						    //
// DATE:	03/05/96						    //
// SYNOPSIS:	Header for assembly fader routines			    //
// NOTES:								    //
// ------------------------------------------------------------------------ //

#ifndef _ASMFADER_H
#define _ASMFADER_H

#ifdef __cplusplus
extern "C" {            /* Assume C declarations for C++ */
#endif

//
// Fading routines take a pointer to a source and destination bitmap (DIB data)
// and a pointer to a BITMAPINFOHEADER structure (if 2 DIBs, both must be of
// identical format), and a fading level from 0 to 31. Zero (0) means do not
// fade at all (just copy and/or pixel-double); thirty-one (31) means full
// fade (destination bitmap filled with black). All other values arithmetically
// darken each pixel by the ratio of (LEVEL / 31). Note that this is a
// linear fade of RGB values, without consideration for gamma correction or
// your non-linear eye. Tough toogies.
//
// - ALSO - for the ATARI version of this code, these routines assume (read
// DEMAND) that the source bitmap be exactly 320 pixels wide; and that the
// destination bitmap for the pixel-doubling version be exactly 640 pixels
// wide.
//

// Routine for simultaneously fading and copying a DIB (can fade in place
// by passing identical src & dest pointers)
void FadeAndCopyDIB(LPVOID src, LPVOID dest, UINT height, UINT fadeLevel);

// Routine for simultaneously fading and pixel-doubling a DIB. The height in
// this case is the height of the source (smaller) bitmap. Both bitmaps must
// be in the same format (i.e. color depth), and the second bitmap must be
// EXACTLY twice as wide and at least twice as tall as the original image.
// You could, in theory, fade "in place" by drawing the source image into
// the upper left-hand quadrant of the big DIB, pass the address of the big
// DIB's bits in both "src" and "dest", and still giving the height of the
// smaller bitmap.
void FadeAndPixelDoubleDIB(LPVOID src, LPVOID dst, UINT height, UINT fadeLevel);

#ifdef __cplusplus
}
#endif

#endif

