/********************************************************************\

	Direct Sound Module
	by Rudy Mathieu

\********************************************************************/

#include "dsnd.h"
#include "RJMFile.h"
#include "RJMFileT.h"
#include "Sptypes.h"

LPDIRECTSOUND dSound;

// Creates a Direct Sound Secondary Buffer
// pcmwf = Waveform information
// size_in_secs = size of buffer in seconds
BOOL createDSB(LPDIRECTSOUNDBUFFER *newbuffer,WAVEFORMATEX *pcmwf ,DWORD size)
{
   DSBUFFERDESC desc;
   memset(&desc,0,sizeof(desc));
   desc.dwSize = sizeof(desc);
   desc.dwFlags = DSBCAPS_CTRLDEFAULT;
   desc.dwBufferBytes = size;// << 10;
   desc.lpwfxFormat = pcmwf;
   if (dSound->CreateSoundBuffer(&desc,newbuffer,NULL)==DS_OK)
   {
      return TRUE;
   }
   newbuffer = NULL;
   return FALSE;
}

// Return offset of new data in buffer else returns INVALID_VALUE
DWORD __fastcall DSoundBuffer::FillSoundBuffer(LPVOID ptr,DWORD size)
{
   LPVOID p1,p2;
   DWORD  s1,s2;
   HRESULT err;

   if (snd==NULL || ptr==NULL || WriteBufferAvail(size)==FALSE)
      return INVALID_VALUE;
      
   err=snd->Lock(writePtr,size,&p1,&s1,&p2,&s2,0);
   if (err==DSERR_BUFFERLOST)
   {
	   snd->Restore();
	   err=snd->Lock(writePtr,size,&p1,&s1,&p2,&s2,0);
   }
   if (err==DS_OK) /* Locked ok */
   {
      memcpy(p1,ptr,s1);
	  if (p2)
	  {
	     memcpy(p2,(PBYTE)ptr + s1,s2);
	  }
      if (snd->Unlock(p1,s1,p2,s2)==DS_OK)
	  {
		  return UpdateWritePointer(size);/* Returns old write pointer */
	  }
   }
   return INVALID_VALUE;
}

DSoundBuffer::~DSoundBuffer()
{
	Close();
}

DSoundBuffer::DSoundBuffer()
{
	writePtr = 0;
	bufferSize = 0;
	snd = NULL;
}

DSoundBuffer::DSoundBuffer(WAVEFORMATEX *pcmwf,DWORD size)
{
	snd=NULL;
	bufferSize = 0;
	if (Open(pcmwf,size)==FALSE)
	{
		throw 1;
	}
}

BOOL DSoundBuffer::Open(WAVEFORMATEX *pcmwf,DWORD size)
{
	Close();
	if (size)
		bufferSize = size;

	if (bufferSize==0)
		return FALSE;

    if (createDSB(&snd,pcmwf,bufferSize)==FALSE)
    {
		return FALSE;
    }
	DSBCAPS  sndCaps;
	sndCaps.dwSize = sizeof(sndCaps);
	snd->GetCaps(&sndCaps);
	bufferSize = sndCaps.dwBufferBytes;
	Reset();
	ClearBuffer();
	return TRUE;
}

void DSoundBuffer::Close(void)
{
	if (snd)
	{
		Stop();
		snd->Release();
		snd=NULL;
		writePtr = 0;
	}
}

BOOL DSoundBuffer::ClearBuffer(DWORD size,BYTE value)
{
   LPVOID p1,p2;
   DWORD  s1,s2;
   HRESULT err;
   
   if (size==0 || size>bufferSize)
      size = bufferSize;
   
   err=snd->Lock(writePtr,size,&p1,&s1,&p2,&s2,0);
   if (err==DSERR_BUFFERLOST)
   {
	   snd->Restore();
	   err=snd->Lock(writePtr,size,&p1,&s1,&p2,&s2,0);
   }
   if (err==DS_OK) /* Locked ok */
   {
      memset(p1,value,s1); /* Clear buffer */
	  if (p2)
	  {
		 memset(p2,value,s2); /* Clear buffer */
	  }

      if (snd->Unlock(p1,s1,p2,s2)==DS_OK)
		  return TRUE;
   }
   return FALSE;
}

#if 0
// Create DSound Object from RAV header info
BOOL __fastcall DSoundBuffer::InitFromRAV(char *filename,DWORD size)
{
	HANDLE fh=Open2Read(filename,FILE_ATTRIBUTE_NORMAL);
	if (fh==INVALID_HANDLE_VALUE)
		return FALSE;	// Error opening file

	FileInfoStr header;

	BOOL err=ReadDatatype(fh,&header);
	CloseHandle(fh);
	if (err==TRUE)
	{
		WAVEFORMATEX wf;
		memset(&wf,0,sizeof(wf));
		if (!memcmp(&wf,&header.waveInfo,sizeof(wf)))
		{	// No audio track
			return FALSE;
		}
		return Open(&header.waveInfo,size);
	}
	return FALSE;
}
#endif
