/****************************************************\
	DIB for video fader control.
	2 DC's are opened all the time!!!
	Is this legal. I sure hope so.
\****************************************************/
#include "Fader.h"

#if FADER_DIBBITDEPTH == 24
/*------------------------------------------------*\
	These defines are for masking 24 bit 
	color data and handling 24 bit DIB

    There are 256 intensities
\*------------------------------------------------*/

// Fixed point 8.8
#define FIXEDMULT(a,b)	(((a) * (b)) >> 8)

#define MAX_INTENSITIES 256

BOOL 	FaderTableInitialized=FALSE;
BYTE	FaderTable[FADEMAX][MAX_INTENSITIES];

static void SetupFaderTable(void)
{
	if (FaderTableInitialized)
		return;

	for (int i=0; i<FADEMAX; i++)
	{
		for (int t=0; t<MAX_INTENSITIES; t++)
		{	
			FaderTable[i][t]=(BYTE)FIXEDMULT(i,t);
		}
	}
	FaderTableInitialized = TRUE;
}

// level is 0-Black  256-Opaque
//void __fastcall FaderControl::UseFilter(int level, HDC destDC)
{
	if (level>=FADEMAX || level<1)
		return;

	PBYTE src = (PBYTE)DIBBuffer1;
	PBYTE dest= (PBYTE)DIBBuffer2;

	int width = DIBWidth * DIBHeight * 3;
	for (int i=0; i<width; i++)
	{
		*(dest++) = FaderTable[level][(*src++)];
	}

	BitBlt(destDC,0,0,DIBWidth,DIBHeight,DIBdc2,0,0,SRCCOPY);
}

#else
/*------------------------------------------------*\
	These defines are for masking 16 bit 1555
	color data and handling 16 bit DIB

    There are 32 intensities and data 11.5 FP
\*------------------------------------------------*/
#define RED_MASK(p)		((p) & (0x001f << 10)) >> 10
#define GREEN_MASK(p)	((p) & (0x001f << 5)) >> 5
#define BLUE_MASK(p)	((p) & (0x001f))

// Fixed point 11.5
#define FIXEDMULT(a,b)	(((a) * (b)) >> 5)

#define MAX_INTENSITIES 32

BOOL 	FaderTableInitialized=FALSE;
WORD	FaderTableR[FADEMAX][MAX_INTENSITIES];
WORD	FaderTableG[FADEMAX][MAX_INTENSITIES];
WORD	FaderTableB[FADEMAX][MAX_INTENSITIES];

static void SetupFaderTable(void)
{
	if (FaderTableInitialized)
		return;

	for (int i=0; i<FADEMAX; i++)
	{
		for (int t=0; t<MAX_INTENSITIES; t++)
		{	
			FaderTableR[i][t]=(WORD)FIXEDMULT(i,t) << 10;
			FaderTableG[i][t]=(WORD)FIXEDMULT(i,t) << 5;
			FaderTableB[i][t]=(WORD)FIXEDMULT(i,t);
		}
	}
	FaderTableInitialized = TRUE;
}
#include "asmfader.h"
// level is 0-Black  32-Opaque
void __fastcall FaderControl::UseFilter(int level, int zoom, HDC destDC)
{
#if 1
  if (zoom==1)
  {
    FadeAndCopyDIB(DIBBuffer1, DIBBuffer2, DIBHeight, 31-level);
  }
  else
  {
    FadeAndPixelDoubleDIB(DIBBuffer1, DIBBuffer2, DIBHeight/2, 31-level);
  }
#else
#if 1
	// Drop every other frame
	static UINT dropframe;
	if (dropframe++ & 1)
	{
		return;
	}
#endif

	if (level>=FADEMAX || level<1)
		return;

	PWORD src = (PWORD)DIBBuffer1;
	PWORD dest= (PWORD)DIBBuffer2;

	int width = DIBWidth * DIBHeight;
	for (int i=0; i<width; i++)
	{
		*(dest++) = 
			FaderTableR[level][RED_MASK(*src)]		|
			FaderTableG[level][GREEN_MASK(*src)]	|
			FaderTableB[level][BLUE_MASK(*src)]	;

		src++;
	}
#endif
	BitBlt(destDC,0,0,DIBWidth,DIBHeight,DIBdc2,0,0,SRCCOPY);

}

#endif

BOOL FaderControl::Open(int width, int height)
{
  SetupFaderTable();

  Close();
  DIBWidth=width;
  DIBHeight=height;

  BITMAPINFOHEADER bh1;
  BITMAPINFOHEADER bh2;

  HDC tempDC = CreateDC("DISPLAY",NULL,NULL,NULL);
  DIBdc1 = CreateCompatibleDC(tempDC);
  DIBdc2 = CreateCompatibleDC(tempDC);
  DeleteDC(tempDC);

  // create a dib section for containing the bits
  memset(&bh1, 0, sizeof(BITMAPINFOHEADER));
  bh1.biSize = sizeof(bh1);
  bh1.biWidth = 320;
  bh1.biHeight = 192;
  bh1.biPlanes = 1;
  bh1.biBitCount = 16;
  bh1.biCompression = BI_RGB;
  bh2 = bh1;
  bh2.biWidth = (width>320)?640:320;
  bh2.biHeight = (width>320)?384:192;

  hBitmap1 = CreateDIBSection( NULL
    , (BITMAPINFO*)&bh1
    , DIB_RGB_COLORS
    , (void**) &DIBBuffer1
    , NULL
    , 0 
    );
  hBitmap2 = CreateDIBSection( NULL
    , (BITMAPINFO*)&bh2
    , DIB_RGB_COLORS
    , (void**) &DIBBuffer2
    , NULL
    , 0 
    );

  if (hBitmap1==NULL || hBitmap2==NULL)
  {
    Close(); // Error
    return FALSE;
  }

  SelectObject(DIBdc1,hBitmap1);
  SelectObject(DIBdc2,hBitmap2);

  return TRUE;
}

void FaderControl::Close(void)
{
	if (DIBdc1)
	{
		DeleteDC(DIBdc1);
		DIBdc1=NULL;
	}

	if (DIBdc2)
	{
		DeleteDC(DIBdc2);
		DIBdc2=NULL;
	}

	if (hBitmap1)
	{
		DeleteObject(hBitmap1);
		hBitmap1 = NULL;
	}

	if (hBitmap2)
	{
		DeleteObject(hBitmap2);
		hBitmap2 = NULL;
	}
}
