/********************************************************************\

	Film Class Module
	by Rudy Mathieu

\********************************************************************/

#ifndef _FilmH
#define _FilmH

#include <windows.h>
#include "linker.h"
#include "render.h"
#include "dsnd.h"
#include "sptypes.h"
#include "shandle.h"
#include "sreader.h"
#include "RAVOut.h"

typedef enum { 

	NOREASON = 0,
	FILMDONEPLAYING,

} ReasonForCB;


#define		SSB_SIZE	0x10000		// Static Sample Buffer

class Film;

typedef void (*FilmCallback)(Film *, LPVOID);
typedef void (*RMCallbackfn)(Film *, LPVOID, ReasonForCB);


/* Used as structure of triggers */
typedef struct {

	DWORD 				frame;	/* Frame to hook */
	FilmCallback		fn;		/* Callback Function */
	LPVOID 				user;   /* User Information */
	
} EventStr, *PEventStr;

class Film //: public RAVSampleReader
{
  RAVSampleReader reader;
  FileInfoStr     header;
	RAVOutput			*ravout;		/* Output */
//	SmartGroup			*shandle;		/* Smart Handle */
//	int					shandleid;		/* Smart Handle ID */

	PFrameSampleStr		sample;

    DWORD 				frame;			/* Current Frame */
	int 				loop;			/* Loops to go */
	int 				loopCount;		/* Current Loop */

	// Static triggers in case no triggers are assigned
//	TLINK<EventStr>		staticTriggers;	

	// Custom triggers
	TLINK<EventStr>		*trigger;

	RMCallbackfn		rmCallback;		/* Resource managment callback */
	LPVOID				rmParam;		/* Parameter for callback */

	BOOL				statusPAUSE;
	BOOL				statusPLAYING;
	BOOL				statusVIDEODIFFERENT;
	BOOL				statusNO_AUDIO_TRACK;
	BOOL				statusSAMPLE_READY;
	BOOL				statusALL_WILL_FIT;

	PBYTE				staticSampleBuffer[SSB_SIZE];

	FADETYPE			clipfadetype;
	int					clipfadeinc;

	void	Init(void);
	void	CheckTriggers(void);
	BOOL	CheckSoundFormat(void);
	int		ReadNextFrame(void);
	BOOL	Looping(void);
	int		SetupFilm(void);

	void Fade(void) { Fade(clipfadetype,clipfadeinc); }

   // Prevent assignments and copies.
   Film& operator=(const Film& f)
   {
     return *this;
   }
   Film(const Film& t)
   {
   }
public:
	~Film();
	Film();
	
	void	__fastcall Setup(RAVOutput *r, SmartGroup *sh)
	{ 
		ravout = r; reader.Link(sh);// shandle = sh;
	}
	BOOL	__fastcall Open(int id);
	void	Close(void);

	BOOL	Play(LONG loopcount=1,FADETYPE type=NOFADE, int incr=1);
	void	PlayToEnd(void) { loop = 1; }
	void	Stop(void) 
	{ 
		statusPLAYING=FALSE; 
		if (ravout)
			ravout->Sound()->Stop(); 
	}

	void	Pause(void) { statusPAUSE=TRUE; ravout->Pause(); }
	void	Resume(void) { statusPAUSE=FALSE; ravout->Resume(); }

	//BOOL	AllWillFit(void) { return statusALL_WILL_FIT; }
	BOOL	Playing(void) { return statusPLAYING; }
	BOOL	Paused(void) { return statusPAUSE; }
	
	DWORD 	GetFrame(void) { return frame; }
	DWORD	GetLoopCount(void) { return loopCount; }

	void	SetVolume(LONG volume) { ravout->Sound()->SetVolume(volume); }
	void	SetPanPosition(LONG pan) { ravout->Sound()->SetPanPosition(pan); }
	LONG	GetVolume(void) { return ravout->Sound()->GetVolume(); }
	LONG	GetPanPosition(void) { return ravout->Sound()->GetPanPosition(); }
		
	void	SetupTriggers(TLINK<EventStr> *trigs) { trigger = trigs; }
	int		AddCallback(DWORD frame, FilmCallback fn, void *user);
	int		RemoveCallback(int h);

   BOOL ReaderStatusLoaded()
   {
     return reader.StatusLoaded();
   }

   // Film's heartbeat
   BOOL Check(void); /* To be run by Timer Service */

   // Sets up resource management callbacks
   void SetupRMCallback(RMCallbackfn fn, LPVOID param)
   {
     rmCallback = fn;
     rmParam = param;
   }

   void __fastcall Fade(FADETYPE ft, int inc)
   {
     if (ravout)
       ravout->Display()->Fade(ft,inc,ravout->Sound());
   }

   int GetTotalFrames() const { return header.totalframes; }
};

#endif
