#include "fwl5.h"
#include "regkey.h"
#include <assert.h>

RegKey::RegKey(HKEY par, const char* sub)
  : parent(par)
  , subKey(sub)
  , myKey(0)
{
}
RegKey::~RegKey()
{
  if (myKey)
  {
    RegCloseKey(myKey);
  }
}
int RegKey::Opened() const { return myKey != 0; }
void RegKey::Open()
{
  long ec = RegOpenKeyEx( parent
                        , subKey
                        , 0
                        , KEY_ALL_ACCESS
                        , &myKey
                        );
  if (ec != ERROR_SUCCESS)
  {
    myKey = 0;
  }
}
void RegKey::Create()
{
  myKey = 0;
  DWORD outcome;
  RegCreateKeyEx( parent
                , subKey
                , 0 // reserved
                , ""// class string
                , REG_OPTION_NON_VOLATILE // special options
                , KEY_WRITE // access
                , 0 // security
                , &myKey
                , &outcome // disposition (as opposed to result?)
                );
}
void RegKey::SetValue(const char* name, const char* value)
{
  RegSetValueEx(myKey
               , name
               , 0
               , REG_SZ
               , (LPBYTE)value
               , strlen(value)+1
               );
}
string RegKey::GetValue(const char* name)
{
  char buf[256];
  DWORD itsType = REG_SZ;
  DWORD bufSize = sizeof(buf);
  long ec = ~ERROR_SUCCESS;
  // Note: On Win95, this function threw exceptions during my
  // debugging UNTIL I got the parameters exactly right.
  ec = RegQueryValueEx( myKey
                      , name
                      , 0
                      , &itsType
                      , (LPBYTE)buf
                      , &bufSize
                      );
  if (ec == ERROR_SUCCESS)
  {
    return buf;
  }
  return "";
}

void RegKey::SetValue(const char* name, const char* value,int valueSize)
{
  RegSetValueEx( myKey
               , name
               , 0
               , REG_BINARY
               , (LPBYTE)value
               , valueSize
               );
}

void RegKey::GetValue(const char* name, char* valueDest,int valueSize)
{
  DWORD itsType = REG_BINARY;
  DWORD bufSize = valueSize;  // has to be a DWORD
  long ec = ~ERROR_SUCCESS;
  // Note: On Win95, this function threw exceptions during my
  // debugging UNTIL I got the parameters exactly right.
  ec = RegQueryValueEx( myKey
                      , name
                      , 0
                      , &itsType
                      , (unsigned char*)valueDest
                      , &bufSize
                      );
  if (ec != ERROR_SUCCESS)
  {
    memset(valueDest, 0, valueSize);
  }
}

DWORD RegKey::GetDWORD(const string& name)
{
  DWORD itsType = REG_DWORD;
  DWORD result;
  DWORD bufSize = sizeof(result);  // has to be a DWORD
  long ec = ~ERROR_SUCCESS;
  // Note: On Win95, this function threw exceptions during my
  // debugging UNTIL I got the parameters exactly right.
  ec = RegQueryValueEx( myKey
                      , name.c_str()
                      , 0
                      , &itsType
                      , (unsigned char*)&result
                      , &bufSize
                      );
  if (ec != ERROR_SUCCESS)
  {
    result = 0;
  }
  return result;
}

string RegKey::GetSubKey(int index)
{
  DWORD rsize=128;
  DWORD reserved=0;
  string buf; buf.resize(rsize);
  if (ERROR_SUCCESS != RegEnumKeyEx
        ( myKey      // handle of key to enumerate 
        , index         // index of subkey to enumerate 
        , (char*)buf.c_str()  // address of buffer for subkey name 
        , &rsize    // address for size of subkey buffer 
        , &reserved // reserved 
        , 0         // address of buffer for class string 
        , 0         // address for size of class buffer 
        , 0         // address for time key last written to
        )
      )
  {
    buf = "";
  }
  return buf;
}
