#ifndef __BASEWIN_H
#define __BASEWIN_H
#include "crect.h"
class Bitmap;
class Palette;
#include "cdc.h"
#include "cstring.h"

struct CmdAssoc; // forward

class BaseWin
{
  public:
	string GetTitle();
	void SetItemText(int item, const string& text);
	string GetItemText(int item, int len=128);
	void CheckRadioButton(UINT id1, UINT id2, UINT idcheck);
	void DisableItem(UINT id);
	void EnableItem(UINT id);
	BOOL GetItemCheck(UINT id);
	void SetItemCheck(UINT id, BOOL state);
	LRESULT SendItemMessage(UINT id, UINT msg, WPARAM wParam, LPARAM lParam);
	LRESULT SendMessage(UINT msg, WPARAM wParam, LPARAM lParam);
	virtual BOOL OnContextMenu(HWND h, const CPoint& where);
	virtual BOOL Create();
	void SetDlgMsgResult(DWORD res);
    virtual ~BaseWin();
    void AddChild(BaseWin* newGuy);
    void Destroy();
    void Disable();
    void Enable();
    BaseWin* GetItem(int id);
    HWND GetHWND() const;
    int GetCheck() const;
    CRect GetOuterRect() const
    {
      return outerRect;
    }
    void Move(const CRect& dest)
    {
      outerRect = dest;
      if (hwnd) 
      {
        MoveWindow( hwnd
                  , outerRect.left
                  , outerRect.top
                  , outerRect.GetWidth()
                  , outerRect.GetHeight()
                  , TRUE
                  );
      }
    }
    CRect GetInnerRect() const
    {
      RECT r;
      GetClientRect(hwnd, &r);
      return r;
    }
    void Hide();
    void InvalidateArea(const CRect& invalid);
    void InvalidateAll();
    void SetCheck(int state);
    void SetFocus();
    void SetFocus(HWND h);
    void SetHWND(HWND h);
    void SetStyle(long newstyle)
    {
      style = newstyle;
      if (hwnd) SetWindowLong(hwnd, GWL_STYLE, style);
    }
    void SetID(int _id)
    {
      id = _id;
    }
    void SetTitle(const string& title);
    void SetTitle(const char* title);
    void UpdateArea(const CRect& update);
    void UpdateAll();
    void Show();
    
    virtual BOOL OnCreate() { return TRUE; }
    virtual void OnDestroy() {}
    virtual void OnSize() {}
    virtual void MouseDown(const CPoint& place, int flags) {}
    virtual void MouseMove(const CPoint& place, int flags) {}
    virtual int  MouseUp(const CPoint& place, int flags) { return 0; }
    virtual void MouseDbl(const CPoint& place, int flags) {}
    virtual void Timer() {}
    virtual void OnPaint(const CDC& h, const CRect& update) {}

    virtual void Key(int code, int flags) {}

    virtual int Idle() { return TRUE; }

    virtual BOOL QueryClose() { return TRUE; }

    //virtual void SetPalette(Palette&) {}
    //virtual Palette GetPalette() const;

    virtual void Execute(int id, int note);
    virtual BOOL WantsDialogKeys() { return TRUE; }

  protected:
    BaseWin()
      : hwnd(0)
      , id(0)
      , exStyle(0)
    {
    }
    BaseWin(HWND par, int id_)
      : hwnd(0)
      , id(id_)
      , exStyle(0)
    {
      hwnd = GetDlgItem(par, id_);
      if (hwnd)
      {
        style = GetWindowLong(hwnd, GWL_STYLE);
      }
    }
    BaseWin(int i, const char* s, const CRect& r, long sl)
      : id(i)
      , title(s)
      , outerRect(r)
      , style(sl)
      , exStyle(0)
      , hwnd(0)
    {
    }

    BaseWin(int i, const char* s, const CRect& r, long sl, long esl)
      : id(i)
      , title(s)
      , outerRect(r)
      , style(sl)
      , exStyle(esl)
      , hwnd(0)
    {
    }
    void SeekAndExecute(CmdAssoc*, int, int);
    void SetExStyle(DWORD val);
    DWORD exStyle;
		string title;
		CRect outerRect;
		int id;
    long style;
    inline LRESULT SendMessage(HWND other, UINT msg, WPARAM w, LPARAM l)
    {
      return ::SendMessage(other, msg, w, l);
    }
  private:
		HWND hwnd;
};

                    /*
//-----------------------------------------------------------------------------
inline void BaseWin::SetCheck(int state)
{
  SendMessage(hwnd, WM_SETCHECK, state, 0);
}
//-----------------------------------------------------------------------------
inline int BaseWin::GetCheck(int id)
{
  return SendMessage(hwnd, WM_GETCHECK, 0, 0);
}
*/
inline void BaseWin::SetFocus()
{
  ::SetFocus(hwnd);
}
inline void BaseWin::SetFocus(HWND h)
{
  ::SetFocus(h);
}
inline void BaseWin::SetTitle(const string& title)
{
  SetTitle(title.c_str());
}

inline void BaseWin::Enable()
{
  EnableWindow(hwnd, TRUE);
}

inline void BaseWin::Disable()
{
  EnableWindow(hwnd, FALSE);
}

inline CWindowDC::CWindowDC(BaseWin* w)
: CDC(0)
{
  h2 = w->GetHWND();
  h = GetDC(h2);
}

struct CmdAssoc { int key; void (BaseWin::*fn)(int, int); };

#define DECLARE_COMMAND_LOOKUP() \
  public: \
    void Execute(int cmd, int note) \
    { \
      SeekAndExecute(cmdTable, cmd, note); \
    }   \
  private: \
    static CmdAssoc cmdTable[]

#define BEGIN_COMMAND_TABLE(className) \
CmdAssoc className::cmdTable[] = {

#define END_COMMAND_TABLE() { 0, 0 }}

#define BIND_COMMAND(className, fnName, key) \
{ key, (void (BaseWin::*)(int,int))&className::fnName },


#endif
