#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "dibsect.h"

//----------------------------------------------------------
DibSection::DibSectionRep::DibSectionRep
  ( BYTE* source
  , const CSize& size
  , Type type
  )
{
  struct fullBitmap
  {
    BITMAPINFOHEADER bi;
    RGBQUAD colors[256];
  };
  fullBitmap header;
  header.bi.biSize = sizeof(header.bi);
  header.bi.biWidth = size.cx;
  header.bi.biHeight = size.cy;
  switch (type)
  {
  case T565:
    {
      DWORD* masks = (DWORD*)&header.colors[0];
      masks[0] = RED565;
      masks[1] = GRN565;
      masks[2] = BLU565;
      header.bi.biCompression = BI_BITFIELDS;
      header.bi.biPlanes = 1;
      header.bi.biBitCount = 16;
      bytesPerRow = ((size.cx*16)+31)/32*4;
    }
    break;
  case T555:
    header.bi.biCompression = BI_RGB;
    header.bi.biBitCount = 16;
    header.bi.biPlanes = 1;
    bytesPerRow = ((size.cx*16)+31)/32*4;
    break;
  case T8:
    header.bi.biCompression = BI_RGB;
    header.bi.biBitCount = 8;
    header.bi.biPlanes = 1;
    bytesPerRow = ((size.cx*8)+31)/32*4;
    break;
  }// switch
  header.bi.biSizeImage = 0; // because Win32 knows.
  header.bi.biXPelsPerMeter = 0; //
  header.bi.biYPelsPerMeter = 0; //
  header.bi.biClrUsed = 0;
  header.bi.biClrImportant = 0;
  HDC basis = ::GetDC(0);
  bitmapDC = CreateCompatibleDC(basis);
  bitmap = CreateDIBSection( basis
                           , (BITMAPINFO*)&header
                           , DIB_RGB_COLORS
                           , (void**)&data
                           , 0 // not from a file
                           , 0 // so no offset
                           );
  ReleaseDC(0, basis);
  bitmap = (HBITMAP)SelectObject(bitmapDC, bitmap);
  if (source)
  {
    CopyMemory(data, source, bytesPerRow*size.cy);
  }
}
//----------------------------------------------------------
DibSection::DibSectionRep::~DibSectionRep()
{
  bitmap = (HBITMAP)SelectObject(bitmapDC, bitmap);
  DeleteObject(bitmap);
  DeleteDC(bitmapDC);
}
//----------------------------------------------------------
void DibSection::DibSectionRep::Blit
  ( const CDC& destDC
  , const CPoint& dest
  , const CRect& src
  ) const
{
  BitBlt( destDC.Get()
        , dest.x, dest.y
        , src.GetWidth(), src.GetHeight()
        , bitmapDC
        , src.left, src.top
        , SRCCOPY
        );
}
//----------------------------------------------------------
void DibSection::DibSectionRep::StretchBlit
  ( const CDC& destDC
  , const CRect& dest
  , const CRect& src
  ) const
{
  StretchBlt( destDC.Get()
            , dest.left, dest.top
            , dest.GetWidth(), dest.GetHeight()
            , bitmapDC
            , src.left, src.top
            , src.GetWidth(), src.GetHeight()
            , SRCCOPY
            );
}
