#ifndef __FILECLIP_H
#define __FILECLIP_H
//-----------------------------------------------------------------------------
class PromptManager;
class NavController;
enum NCError;
class CEvent;
class Scheduler;
class Film;
class VoiceOverManager;
#include "sptypes.h" // I only need fadetypes... BUT NOOOOO!
#include "cpnodes.h"
#include "cpnet.h"
#include "pstate.h"
//-----------------------------------------------------------------------------
struct Scheduled // link between an CEvent from scheduler and a DB node.
{
  CEvent* event;
  CPNode* node;
};

//-----------------------------------------------------------------------------

class FileManager
{
public:
  FileManager()
    : controller(0)
    , curClip(0)
    , curNode(0)
    , loadIt(0)
    , paused(0)
    , promptManager(0)
    , reminderPointer(playerState)
    , reminding(FALSE)
    , scheduler(0)
    , bigfileHD(0)
    , voiceOverManager(0)
    , currentDisk(0)
    , newDisk(0)
  {
    it = this;
  }
  ~FileManager()
  {
  }
  void ChangeCD(int newCD);
  BOOL DifferentDiskRestart(int& newDisk);
  BOOL DifferentDiskResume(int& newDisk);
  BOOL DifferentDiskReminder(int &newDisk);
  BOOL DifferentDiskRetry(int& newDisk);
  void DoReminder();
  void DoRetry();
  string GetAnalysis() const;
  const Psych& GetPsych() const;
  long GetScore() const;
  BOOL IsOver();
  BOOL IsPaused();
  void Manage();
  void PausePlayback();
  NCError RestoreGame(const char* n);
  void ResumePlayback();
  BOOL SaveGame(const char* n) const;
  void SetArchive(FRSRC* bigfile);
  void StartOver();
  void TakeABranch();
  void TalkTo(NavController* cont)
  {
    controller = cont;
  }
  void TalkTo(PromptManager* pr)
  {
    promptManager = pr;
  }
  void TalkTo(Scheduler* s)
  {
    scheduler = s;
  }
  void TalkTo(VoiceOverManager* v)
  {
    voiceOverManager = v;
  }
private:
  void AddReminderClips();
  void AddClipsToPlay();
  CEvent* AddOneFile(CPNode& n, FADETYPE f);
  CEvent* AddReminderFile(CPNode& n, FADETYPE f);
  BOOL BranchEnabled(int kb);
  void DisableBranch(int k);
  void EnableAllBranches();
  char *GetFileName(const char* base);
  PlayerStateIterator GetReminderStart() const;
  void OnStartClip(Film* f, CPNode* n);
  void OnEndClip(Film*,CPNode*);
  void SegmentOff(Film* f, CPNode* node, BOOL loop, int kSegment);
  void SegmentOn(Film* f, CPNode* node, BOOL loop, int kSegment);
  void WatchSegments();
  static void HyperOff(Film* f, void* n);
  static void HyperOn(Film* f, void* n);
  static void KillLeadin(Film*, void*);
  static void KillLoop(Film*, void*);
  static void KillScheduled(Film*, void*);
  static void KillReminder(Film*,void*);
  static void ResetLeadin(Film*, void*);
  static void ResetLoop(Film*, void*);
  static void ResetScheduled(Film*, void*);
  static void ResetReminder(Film*,void*);
  static void SSegmentOff(Film* f, void* d);
  static void SSegmentOn(Film* f, void* d);
  FRSRC* bigfileHD;
  int currentDisk;
  NavController* controller;
  int disabledBranches;
  CPNode* curNode;
  Film* curClip;
  CPNode* loadIt;
  CPNetwork net;
  int newDisk;
  BOOL paused;
  PlayerState playerState;
  PromptManager* promptManager;
  Psych psych;
  PlayerStateIterator reminderPointer;
  BOOL reminding;
  Scheduler* scheduler;
  long score;
  VoiceOverManager* voiceOverManager;
  static FileManager* it;
};
#endif