#ifndef __FONTBITS_H
#define __FONTBITS_H
#include "cpoint.h"
#include "crect.h"
#include "cstring.h"
#include "csize.h"
#include "dibsect.h"
#include "cdc.h"
// Some test code is different if the font assumed byte data.
#define ITS_WORDS

// If you stick to these colors, you can draw text in color.
// No other colors are going to work.
enum 
{ FONT_WHITE=0xffff
, FONT_RED=0xf800
, FONT_GREEN=0x07e0
, FONT_BLUE=0x001f
, FONT_MAGENTA=0xf81f
, FONT_YELLOW=0xffe0
, FONT_CYAN=0x07ff
, FONT_GRAY=(12<<11)|(24<<5)|12
, FONT_DKGRAY=(3<<11)|(6<<5)|3
, FONT_BLACK=0
};
// FontBitsRep:
// This class is the functional part of the FontBits class.
// It can be initialized from a resource in your .exe.
// The resource must be of type "FONTBITS"
// FontBits:
// This class is an envelope for a reference-counted FontBitsRep.
// This class has the assignment operator and copy constructor
// which manage reference counts.
// The * and -> operators give access to the functionality of the
// FontBitsRep class.
//
class FontBits
{
public:
  class Rep
  {
  public:
  	BOOL GetAllWidths(unsigned* widths, unsigned nWidths) const;
  	unsigned GetFirstAscii() const;
  	unsigned GetNumGlyphs() const;
  	unsigned GetHeight() const;
    Rep(int id)
      : zoom(1)
    {
      SetupResource((LPCTSTR)id);
    }
    Rep(char* name)
      : zoom(1)
    {
      SetupResource(name);
    }
    ~Rep()
    {
    }

    void DrawString(const CDC& ddc, const string& text);
    void DrawString(const CDC& ddc, const string& text, const int* deltas);
    void DrawString(const CDC& ddc, const char* text, const int* deltas, int nChars);

    CSize MeasureString(const string& text) const;
    BOOL GetWidths(const string& text, int* widths) const;
    BOOL GetWidths(const char* text, int* widths, int nChars) const;
    void SetColor(int color);
    int GetColor() const;
    void SetCP(const CPoint& where);
    CPoint GetCP() const;
    void SetZoom(int z) { zoom = (short)z; }
    int GetZoom() const { return zoom; }

    //void DrawCharacter(HDC ddc, const CPoint& dp, int color, int glyph) const;

  private:
  	void ReadyCharacter(int glyph) const;
    unsigned char toglyph(char c) const { return ((c>64)?(c&~32):c)-firstGlyph; }
  	BOOL IsGlyph(char c) const { return (toglyph(c)>=0 && toglyph(c)<font.nGlyphs); }
  	short firstGlyph;
    void SetupResource(LPCTSTR name);
    void DrawCharacter(const CDC& ddc, int glyph) const;

    friend class FontBits;
    int refs;
    CPoint cp;
  
    struct fontHdr 
    { 
      BYTE _type;
      BYTE nGlyphs;
      BYTE cellWidth; 
      BYTE cellHeight;
      BYTE firstAscii;
      BYTE lastAscii;
      BYTE r7;
      BYTE r8;
      WORD data[2]; // actually nGlyphs*cellWidth*cellHeight
    };
  
    fontHdr font;
    BYTE widths[64];

    // To save some rendering time, we cache colored and stretched characters.
    // The cached area of readyDib is used if the color and zoom are the same
    // as they were for that character. If not, the corresponding section of
    // rawDib is stretched and colored and put in readyDib.
    struct cacheTrack
    {
      short zoom;
      short color;
    };
    cacheTrack colorTrack[256];
    short color;
    short zoom;
    DibSection rawDib;
    DibSection readyDib;
  };

  FontBits()
  {
    rep = 0;
  }
  FontBits(int id)
  {
    rep = new Rep(id);
    rep->refs=1;
  }
  FontBits(char* name)
  {
    rep = new Rep(name);
    rep->refs=1;
  }
  FontBits(const FontBits& other)
  {
    rep = other.rep;
    GainOne();
  }
  ~FontBits()
  {
    LoseOne();
  }
  FontBits& operator=(const FontBits& other)
  {
    if (rep!=other.rep)
    {
      LoseOne();
      rep = other.rep;
      GainOne();
    }
    return *this;
  }
  int operator==(const FontBits& other) const
  {
    return rep==other.rep;
  }
  int operator!=(const FontBits& other) const
  {
    return rep==other.rep;
  }
  Rep* operator->()
  {
    return rep;
  }
  Rep* operator->() const
  {
    return rep;
  }
  Rep& operator*()
  {
    return *rep;
  }
private:
  Rep* rep;
  // Release a reference to the pointer we have now.
  void LoseOne()
  {
    if (rep)
    {
      rep->refs--;
      if (rep->refs==0)
      {
        delete rep;
      }
    }
    rep=0;
  }
  // Add a reference to our current pointer.
  void GainOne()
  {
    if (rep)
    {
      rep->refs++;
    }
  }
};

//---------------------------------------------------------------
inline void FontBits::Rep::SetColor
  (int col)
{
  color = (short)col;
}

//---------------------------------------------------------------
inline int FontBits::Rep::GetColor() const
{
  return color;
}

//---------------------------------------------------------------
inline void FontBits::Rep::SetCP
  (const CPoint& where)
{
  cp = where;
}

//---------------------------------------------------------------
inline CPoint FontBits::Rep::GetCP() const
{
  return cp;
}

//---------------------------------------------------------------
inline unsigned FontBits::Rep::GetHeight() const
{
  return font.cellHeight*zoom;
}

//---------------------------------------------------------------
inline unsigned FontBits::Rep::GetNumGlyphs() const
{
  return font.nGlyphs;
}

//---------------------------------------------------------------
inline unsigned FontBits::Rep::GetFirstAscii() const
{
  return font.firstAscii;
}

#endif
