#include "fwl5.h"
#include "gamewin.h"
#include "resource.h"
#include "funkybar.h"
#include "options.h"
#include "navcontr.h"
#include "ddraw.h"
#include "ddbox.h"
#include "dbgtst.h"
#include "commctrl.h"
//#define CHECK_DD
GameWindow::GameWindow(const char* command_)
  : Window()
  , command(command_)
  , menuShowing(FALSE)
  , controller(0)
  , dd(0)
  , primary(0)
  , back1(0)
  , back2(0)
{
  controller = new NavController(command);
  controller->TalkTo(this);
}
GameWindow::~GameWindow()
{
  delete controller;
}

BOOL GameWindow::Create(void)
{
  return (Window::Create() && MakeFunkyBar());
}

void GameWindow::initTest()
{
  DEBUGSTRING(("inittest"));
  controller->InitTest();
}

BOOL GameWindow::MakeFunkyBar(void)
{
  funkyBar = new FunkyBar(this, 20);
  if (funkyBar->Create())
  {
    funkyBar->Message("");
    return TRUE;
  }
  return FALSE;
}
//---------------------------------------------------------------------------
void GameWindow::CheckPrimary()
{
#ifdef CHECK_DD
  DEBUGSTRING(("cp"));
  if (primary)
  {
    if (primary->IsLost())
    {
      primary->Restore();
    }
  }
#endif
}
//---------------------------------------------------------------------------
int GameWindow::Idle()
{
  if (AppIsActive())
  {
    CheckPrimary();
    if (controller->Idle())
    {
      return !AppIsActive();
    }
  }
  //else
  {
    return 1;
  }
}
//---------------------------------------------------------------------------
void GameWindow::MouseDown(const CPoint& place, int flags)
{
  controller->MouseDown(place, flags);
}

//---------------------------------------------------------------------------
void GameWindow::OnExitApp(int,int)
{
  controller->OnAppExit();
  //Destroy();
  Close();
}

//---------------------------------------------------------------------------
void GameWindow::OnDestroy()
{
#ifdef CHECK_DD
  if (dd)
    dd->RestoreDisplayMode();
#endif
	controller->CloseTest();
   delete funkyBar;
	PostQuitMessage(0);
}
//---------------------------------------------------------------------------
void GameWindow::OnPaint(const CDC& dc, const CRect& r)
{
  controller->OnPaint(dc, r);
}
//---------------------------------------------------------------------------
BEGIN_COMMAND_TABLE(GameWindow)
 BIND_COMMAND(GameWindow, OnExitApp, ID_EXITAPP)
 BIND_COMMAND(GameWindow, OnPopup, ID_HELLO_SAVE)
 BIND_COMMAND(GameWindow, OnPopup, ID_HELLO_RESTORE)
 BIND_COMMAND(GameWindow, OnPopup, ID_HELLO_OPTIONS)
 BIND_COMMAND(GameWindow, OnPopup, ID_HELLO_REMINDER)
 BIND_COMMAND(GameWindow, OnPopup, ID_HELLO_RESUME)
 BIND_COMMAND(GameWindow, OnPopup, ID_HELLO_SHOWSTATUS)
 BIND_COMMAND(GameWindow, OnPause, ID_PAUSE)
 BIND_COMMAND(GameWindow, OnGo, ID_HELLO_GO)
END_COMMAND_TABLE();
//---------------------------------------------------------------------------
void GameWindow::OnGo(int,int)
{
  controller->DoGoCommand(); // terminate splash screen and go.
}
//---------------------------------------------------------------------------
void GameWindow::OnPopup(int cmd,int)
{
  switch(cmd)
  {
  case ID_HELLO_SAVE:
    controller->DoSaveCommand();
    break;
  case ID_HELLO_RESTORE:
    controller->DoRestoreCommand();
    break;
  case ID_HELLO_OPTIONS:
    controller->DoOptionsCommand();
    break;
  case ID_HELLO_REMINDER:
    controller->DoReminderCommand();
    break;
  case ID_HELLO_RESUME:
    funkyBar->ChangeButton(FunkyBar::FB_PAUSE);
    controller->DoUnpauseCommand();
    break;
  case ID_HELLO_SHOWSTATUS:
    controller->DoStatusCommand();
    break;
  }
}
//---------------------------------------------------------------------------
void GameWindow::Key(int code, int flags)
{
  controller->Key(code, flags);
}
BOOL GameWindow::WantsDialogKeys()
{
  return controller->WantsDialogKeys();
}
//---------------------------------------------------------------------------
BOOL // return TRUE if you handled the event
  GameWindow::OnContextMenu
    ( HWND h                  // hwnd where the click came from
    , const CPoint& where     // where the click occurred (screen coords)
    )
{
  controller->Pause(); // must pause for this.
  funkyBar->ChangeButton(FunkyBar::FB_PLAY);
  HMENU menu1 = LoadMenu(Application::FWGetAppInstance(), MAKEINTRESOURCE(IDR_MENU1));
  if (menu1)
  {
    HMENU menu = GetSubMenu(menu1, 0);
    UINT flags = TPM_CENTERALIGN | TPM_RIGHTBUTTON;
    HWND xhwnd = GetHWND();

    int kItems = GetMenuItemCount(menu);
    for (int i=0; i<kItems; i++)
    {
      UINT id = GetMenuItemID(menu, i);
      if (!enabledCommands.IsEnabled(id))
      {
        EnableMenuItem(menu,	// handle to menu
            id,	// menu item to enable, disable, or gray
            MF_BYCOMMAND | MF_DISABLED | MF_GRAYED 	// menu item flags
           );	
      }
    }
    SetForegroundWindow(xhwnd);
    if (TrackPopupMenu(menu, flags, where.x, where.y, 0, xhwnd, 0))
    {
      PostMessage(xhwnd, WM_NULL,0,0);
    }
    DestroyMenu(menu1);
  }
  return TRUE;
}

//---------------------------------------------------------------------------
void GameWindow::OnPause(int,int)
{
  CRect toolRect = GetOuterRect();
  if (controller->IsPaused())
  {
    funkyBar->ChangeButton(FunkyBar::FB_PAUSE);
    controller->DoUnpauseCommand();
  }
  else
  {
    funkyBar->ChangeButton(FunkyBar::FB_PLAY);
    controller->Pause();
  }
}

//---------------------------------------------------------------------------
CRect 
  GameWindow::GetClipArea() const
{
  CRect clipRect = GetInnerRect();
  if (clipRect.bottom > 270)
  {
    clipRect.bottom = 384;
  }
  else
  {
    clipRect.bottom = 192;
  }
  return clipRect;
}

//---------------------------------------------------------------------------
CRect 
  GameWindow::GetPromptArea() const
{
  CRect promptRect = GetInnerRect();
  if (promptRect.bottom > 300)
  {
    promptRect.top = 384;
  }
  else
  {
    promptRect.top = 192;
  }
  CRect statusRect = funkyBar->GetOuterRect();
  promptRect.bottom = statusRect.top;
  return promptRect;
}
//------------------------------------------------------
// OtherMessage:
// This is our hook into windows messages.
// The function returns true if it has placed a return
// value in the res param. It should not modify res
// if it returns false.
//
BOOL // false if DefWindowProc should be called
  GameWindow::OtherMessage
    ( UINT msg          // the usual msg number
    , WPARAM wParam     // the usual wParam
    , LPARAM lParam  // the usual lParam
    , LRESULT& res  // what to return in lieu of what would have been.
    )
{
  switch(msg)
  {
  case WM_INITMENU:
    res = OnInitMenu((HMENU) wParam);
    return TRUE;
  case WM_SYSCOMMAND:
    switch (wParam&0xfff0)
    {
    case SC_SCREENSAVE:
      res = 0; // return this
      return TRUE; // I mean it.
    }
    break;
  case WM_HSCROLL:
    // volume slider's been twiddled
    {
      int val = HIWORD(wParam);
      switch(LOWORD(wParam))
      {
      case TB_ENDTRACK:
      case TB_PAGEUP:
      case TB_PAGEDOWN:
        val = SendMessage((HWND)lParam, TBM_GETPOS,0,0);
        break;
      }
      controller->SetVolume(val);
    }
    return FALSE;
  }
  return FALSE;
}
#ifdef CHECK_DD
class BarfWindow : public Window
{
public:
  BarfWindow(Window* par) 
    : Window(par
            ,0   // a window owned by another!
            ,CRect(10,10,210,210)
            ,"barf"
            ,WS_POPUP|WS_CAPTION
            ) 
  {
  }
  void OnMove(int,int)
  {
    DEBUGSTRING(("bwMove\n"));
  }
  void OnSize(int,int,int)
  {
    DEBUGSTRING(("bwSize\n"));
  }
  void OnPaint(const CDC& h, const CRect& update) 
  {
    DEBUGSTRING(("bwPaint\n"));
  }
};
BarfWindow* bwin = 0;
#endif
void GameWindow::SetWindowSize(Size size)
{
  windowState = size;
  switch (size)
  {
  case WZ_ZOOM:
    Move(largeRect);
    funkyBar->Show();
    break;
  case WZ_NORMAL:
    Move(smallRect);
    funkyBar->Show();
    break;
  case WZ_SPLASH:
    Move(splashRect);
    funkyBar->Hide();
    break;
  }
#ifdef CHECK_DD
  if (windowState != WZ_SPLASH)
  {
    if (!bwin)
    {
      bwin = new BarfWindow(this);
      bwin->Create();
      bwin->Show();
    }
    if (!dd)
    {
      dd = new DDBox;
      dd->Create();
      //dd->SetCooperativeLevel(GetHWND(), DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);
      dd->SetCooperativeLevel(bwin->GetHWND(),DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);
      BringWindowToTop(GetHWND());
      DEBUGSTRING(("top changed\n"));
      dd->SetDisplayMode(CSize(640,480), 16);
      DEBUGSTRING(("mode set\n"));
      dd->ShowGDI();
      DEBUGSTRING(("survived showgdi\n"));
//      primary = new DDSurf;
//      DDSurf::Desc primDesc;
//      primDesc.SetPrimary();
//      dd->CreateSurface(primDesc, *primary);
    }
  }
#endif
}

void GameWindow::SetLargeRect(const CRect& rect)
{
  largeRect = rect;
}

void GameWindow::SetSmallRect(const CRect& rect)
{
  smallRect = rect;
}
void GameWindow::SetSplashRect(const CRect& rect)
{
  splashRect = rect;
}
int level=0;
//---------------------------------------------------------------------------
void GameWindow::OnSize(int what, int x, int y)
{
  level++;
#ifdef CHECK_DD
  DEBUGSTRING(("MoveSize level %d ", level));
  CRect r=GetOuterRect();
  CRect i=GetInnerRect();
  char* ws="WZ_NORMAL";
  if (windowState==WZ_ZOOM) ws="WZ_ZOOM";
  if (windowState==WZ_SPLASH) ws="WZ_SPLASH";
  DEBUGSTRING(("Size %s %d %d o(%d,%d,%d,%d) i(%d,%d,%d,%d)\n"
    , ws
    , x, y
    , r.left, r.top, r.right, r.bottom
    , i.left, i.top, i.right, i.bottom
    ));
#endif
  if (what != SIZE_MINIMIZED)
  {
    funkyBar->AutoSize();
    controller->OnSize(x,y);
  }
  level--;
}
//---------------------------------------------------------------------------
void GameWindow::OnMove(int x, int y)
{
  level++;
  CRect r=GetOuterRect();
#ifdef CHECK_DD
  DEBUGSTRING(("MoveSize level %d\n", level));
  CRect i=GetInnerRect();
  char* ws="WZ_NORMAL";
  if (windowState==WZ_ZOOM) ws="WZ_ZOOM";
  if (windowState==WZ_SPLASH) ws="WZ_SPLASH";
  DEBUGSTRING(("Move %s %d %d o(%d,%d,%d,%d) i(%d,%d,%d,%d)\n"
    , ws
    , x, y
    , r.left, r.top, r.right, r.bottom
    , i.left, i.top, i.right, i.bottom
    ));
#endif
  switch (windowState)
  {
  case WZ_ZOOM:
    largeRect = r;
    break;
  case WZ_NORMAL:
    smallRect = r;
    break;
  case WZ_SPLASH:
    splashRect = r;
    break;
  }
  level--;
}

//---------------------------------------------------------------------------
BOOL GameWindow::OnCreate()
{
  controller->TalkTo(this); // !! vexing
  return controller->OnWindowCreate();
}

void GameWindow::StatusMessage(const string& msg)
{
  funkyBar->Message(msg);
}

void GameWindow::ShowMenu()
{
  assert(!menuShowing);
  if (windowState == WZ_SPLASH)
  {
  }
  else
  {
    // locate menu handle
    HMENU menu = LoadMenu(Application::FWGetAppInstance(), MAKEINTRESOURCE(IDR_MENU2));
    // attach menu
    menuShowing = TRUE;
    // make menu draw
    // if currently splash
    //   adjust smallRect
    //   adjust largeRect
    int dh = GetSystemMetrics(SM_CYMENU);
    SetMenu(GetHWND(), menu);
    largeRect.top -= dh;
    smallRect.top -= dh;
    if (windowState == WZ_ZOOM)
    {
      Move(largeRect);
    }
    else
    {
      Move(smallRect);
    }
  }
}

void GameWindow::HideMenu()
{
  assert(menuShowing);
  if (windowState == WZ_SPLASH)
  {
  }
  else
  {
    HMENU was = GetMenu(GetHWND());
    // remove menu
    // make menu draw
    if (was)
    {
      SetMenu(GetHWND(), 0);
      DestroyMenu(was); // cause we detached it, it won't get destroyed automatically.
      CRect outer = GetOuterRect();
      menuShowing = FALSE;
      int dh = GetSystemMetrics(SM_CYMENU);
      largeRect.top += dh;
      smallRect.top += dh;
      if (windowState == WZ_ZOOM)
      {
        Move(largeRect);
      }
      else
      {
        Move(smallRect);
      }
    }
  }
}

void GameWindow::EnableCommand(UINT id)
{
  enabledCommands.Enable(id);
}

void GameWindow::DisableCommand(UINT id)
{
  enabledCommands.Disable(id);
}

BOOL GameWindow::OnInitMenu(HMENU menu)
{
#if 0
  int nDisabled = enabledCommands.GetDisabledCount();
  for (int i=0; i<nDisabled; i++)
  {
    id = enabledCommands.GetDisabledItem();
    EnableMenuItem(menu, id, MF_BYCOMMAND | MF_DISABLED | MF_GREYED);
  }
#endif

  // The policy is to reach out and enable or disable
  // each item. WM_INITMENU is the only time the
  // menubar's items are enabled or disabled, and we
  // only know which are disabled.
  int kItems = GetMenuItemCount(menu);
  for (int i=0; i<kItems; i++)
  {
    UINT id = GetMenuItemID(menu, i);
    if (id == 0xffff) // it's a menu
    {
      HMENU menu2 = GetSubMenu(menu, i);
      int kItems2 = GetMenuItemCount(menu2);
      for (int i2 = 0; i2<kItems2; i2++)
      {
        UINT id2 = GetMenuItemID(menu2, i);
        if (!enabledCommands.IsEnabled(id))
        {
          EnableMenuItem(menu, id, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
        }
        else
        {
          EnableMenuItem(menu, id, MF_BYCOMMAND | MF_ENABLED);
        }
      }
    }
    else
    {
      if (!enabledCommands.IsEnabled(id))
      {
        EnableMenuItem(menu, id, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
      }
      else
      {
        EnableMenuItem(menu, id, MF_BYCOMMAND | MF_ENABLED);
      }
    }
  }
  return TRUE;
}

void GameWindow::DisableResume()
{
  DisableCommand(ID_HELLO_RESUME);
  funkyBar->DisableButton();
}

void GameWindow::EnableResume()
{
  EnableCommand(ID_HELLO_RESUME);
  funkyBar->EnableButton();
}

void GameWindow::PromptForResume()
{
  funkyBar->ChangeButton(FunkyBar::FB_PLAY);
}

void GameWindow::PromptForPause()
{
  funkyBar->ChangeButton(FunkyBar::FB_PAUSE);
}

void GameWindow::IndicateVolume(int idicate)
{
  funkyBar->IndicateVolume(idicate);
}

BOOL GameWindow::QueryClose()
{
  return controller->QueryClose();
}
