//  optionsp.cpp
//  options property sheets
#include "fwl5.h"
#include "resource.h"
#include "propsh.h"
#include "optionsp.h"
#include "options.h"
#include "dialog.h"
#include "dlgitem.h"
#include <assert.h>
#include "pwdialog.h"

extern GameOptions globalOptions;
//char password[51];
//BOOL CALLBACK PasswordDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
////////////////////////////////////////////////////////////////////////////////
int PagePresentation::OnSetActive(CPropertySheet* par)
{
	SetItemCheck(IDC_SUBTITLES,globalOptions.SubtitlesOn());
	SetItemCheck(IDC_VOICEOVERS,globalOptions.VoiceoversOn());
	SetItemCheck(IDC_ZOOMNORMAL,globalOptions.ZoomOn()?0:1);
	SetItemCheck(IDC_ZOOM2X,globalOptions.ZoomOn()?1:0);
	SetItemCheck(IDC_SKIPAHEAD,globalOptions.SkipAheadOn()?1:0);
	SetItemCheck(IDC_NOSKIPAHEAD,globalOptions.SkipAheadOn()?0:1);
	SetItemCheck(IDC_ENTIREPATH,globalOptions.EntirepathOn()?1:0);
	SetItemCheck(IDC_PLAYBACKJUMP,globalOptions.EntirepathOn()?0:1);
	SetItemCheck(IDC_MENUBAR,globalOptions.MenubarOn());
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
int PagePresentation::OnApply(CPropertySheet* par)
{
	globalOptions.SetSubtitles(GetItemCheck( IDC_SUBTITLES));
	globalOptions.SetVoiceovers(GetItemCheck( IDC_VOICEOVERS));
	globalOptions.SetZoom(GetItemCheck( IDC_ZOOM2X));
	globalOptions.SetSkipAhead(GetItemCheck(IDC_SKIPAHEAD));
	globalOptions.SetEntirepath(GetItemCheck(IDC_ENTIREPATH));
	globalOptions.SetMenubar(GetItemCheck(IDC_MENUBAR));
	return PSNRET_NOERROR;
}

////////////////////////////////////////////////////////////////////////////////
int PageParentalControl::OnSetActive(CPropertySheet*)
{
  SetItemCheck(IDC_MINVIOLENCE,globalOptions.MinViolenceOn());
  SetItemCheck(IDC_MINSEX,globalOptions.MinSexOn());
  if((globalOptions.GetPassword()).length()>0)
  {
    DisableItem(IDC_MINVIOLENCE);
    DisableItem(IDC_MINSEX);
        DisableItem(IDC_PWCHANGE);
  }
  else
  {
    EnableItem(IDC_MINVIOLENCE);
    EnableItem(IDC_MINSEX);
        DisableItem(IDC_PWCHANGE);
  }
  return 0;
}

////////////////////////////////////////////////////////////////////////////////
int PageParentalControl::OnApply(CPropertySheet* par)
{
	globalOptions.SetMinViolence(GetItemCheck( IDC_MINVIOLENCE));
	globalOptions.SetMinSex(GetItemCheck(IDC_MINSEX));
	return PSNRET_NOERROR;
}

////////////////////////////////////////////////////////////////////////////////
void PageParentalControl::Execute(int id, int note)
{
  switch(id)
  {
  case IDC_PWCHANGE:
    if (note==BN_CLICKED)
    {
      // run the password setting dialog box
      PWDialog dlg;
      if (dlg.RunModal(this))
      {
        globalOptions.SetPassword(dlg.GetPassword());
        SetItemText(IDC_PASSWORD, "");
        DisableItem(IDC_MINVIOLENCE);
        DisableItem(IDC_MINSEX);
        DisableItem(IDC_PWCHANGE);
      }
    }
    break;
  case IDC_PASSWORD:
    if(note==EN_CHANGE)
    {
      string buffer = GetItemText(IDC_PASSWORD, 100);
      if (buffer==globalOptions.GetPassword())
      {
        EnableItem(IDC_MINVIOLENCE);
        EnableItem(IDC_MINSEX);
        EnableItem(IDC_PWCHANGE);
      }
      else
      {
        DisableItem(IDC_MINVIOLENCE);
        DisableItem(IDC_MINSEX);
        DisableItem(IDC_PWCHANGE);
      }
    }
    break;
  default:
    break;
  }
}

////////////////////////////////////////////////////////////////////////////////
int PageDifficulty::OnSetActive(CPropertySheet*)
{
	int diffLevel = globalOptions.GetDifficultyLevel();
	assert(diffLevel>=0);
	assert(diffLevel<3);
	CheckRadioButton(IDC_DIFF1,IDC_DIFF3,IDC_DIFF1+diffLevel);
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
int PageDifficulty::OnApply(CPropertySheet* par)
{
  int i;
  for(i=0;i<3;i++)
  {
    if(GetItemCheck(IDC_DIFF1+i))
    {
      globalOptions.SetDifficultyLevel(i);
    }
  };
  return PSNRET_NOERROR;
}

