/********************************************************************\

	Scheduler.CPP

\********************************************************************/

#include <windows.h>
#include "Schdlr.h"
#include "stdexcpt.h"
#include "dbgnew.h"
#include "dbgtst.h"

#define SND_SIZE				8		// Must match Ravout.cpp

#define BASE_CDROM_SPEED		420		// Gotta read AHEAD of playback
#define VIDEO_PERIOD			35
#define FILE_PERIOD				((1000*(gChunkSize>>10))/BASE_CDROM_SPEED)

// This way our buffer size aligns on Chunk Boundaries
#define ChunckAlign(x)			(((int)((x<<10)/gChunkSize) * gChunkSize)>>10)
#define NORMAL_BUFFER_SZ		ChunckAlign(480)
#define LOOP_BUFFER_SZ			ChunckAlign(960)

void TriggerPlayNext(Film *film, LPVOID param, ReasonForCB reason)
{
	Scheduler *sch = (Scheduler *)param;
	switch(reason)
	{
	case FILMDONEPLAYING:
		sch->playnext=TRUE;
		break;
	}
}

void OutputChecker(LPVOID param)
{
	((Scheduler *)param)->ravout.DisplayFrame();
}

void Scheduler::Check(void)
{
  if (playnext==TRUE)
    PlayNextEvent();

  if (statusLOOPING)
  {  /* Looping */

    if (user_sel != NONE && 
      user_sel < leadin_count &&
      shandle.Data(leadinID[user_sel])->StatusLoaded())
    {
      /* User selected leadin */

  		// Play loop to end
	  	if (currentEvent) 
		  	currentEvent->PlayToEnd();

  		statusLOOPING = FALSE;
	  	loadfocus = leadinID[user_sel];
		  WipeOtherLeadins();
		  PurgeLeadinsFromList();
      return;
    }

    if (currentEvent->Clip()->ReaderStatusLoaded())//Data()->StatusLoaded())
    {  /* Do only when loop is fully loaded */

      if (leadin_count<NODES-1 && PeekAtNextEvent()==LEADIN) 
      {	/* Leadin Next */
        if (leadin_count) /* Not 1st leadin */
        {
          // Is previous loading leadin fully in buffer?
          if (shandle.Data(leadinID[leadin_count-1])->StatusFull())
          {
            if (LoadNextEvent()==TRUE)
            {
              leadinID[leadin_count++]=loadfocus;
              loadfocus=-1;
            } else {
              // Error while loading next event
              DEBUGSTRING(("LNE failed w/leadin\n"));
            }
          } else {
            /* Leadin buffer still loading */
            /* Do nothing. Simply continue playing loop */
          }	/* nextEvent */
        } else {
          if (LoadNextEvent()==TRUE)
          {
            leadinID[leadin_count++]=loadfocus;
            loadfocus=-1;
          } else {
            // Error while load next event
            DEBUGSTRING(("LNE failed w/leadin\n"));
          }
        } /* leadin_count */
      } /* leadin_count<NODES-1 */
    } /* currentEvent->Clip()->Buffer()->StatusLoaded() */

  } else { /* Then we are not looping */


    /* Play basic shot-type clips */
    if (currentEvent==NULL)
    {  /* Nothing loaded */
      LoadNextEvent(); /* So load something */
      PlayNextEvent();
    } else {
      /* Only cue next if nothing is going on */
      //SmartFile *sf = currentEvent->Clip()->Data();
      //if (loadfocus < 0 && (sf==NULL || sf->StatusLoaded()))
      BOOL clipLoaded = currentEvent->Clip()->ReaderStatusLoaded();
      if (loadfocus < 0 && clipLoaded)
      {	 /* Done loading current and ready to preload next */
        LoadNextEvent();
      }
    }



  } /* status & LOOP_PLAYING */
}

/********************************************************************\
\********************************************************************/

BOOL Scheduler::PlayNextEvent(void)
{
	if (statusLOOPING)
	{
		loadfocus = LoadNextEvent();
	}

	if (currentEvent)
	{
		delete currentEvent;
		currentEvent = NULL;
	}
	
	if (loadfocus < 0)
	{
		// No event loaded to play
		sound.Stop();			// Stop
		sound.ClearBuffer();	// Clear buffer
		ClearLeadinIDs();
		statusPlaying=FALSE;	// Playing flag
		return FALSE;

	} else {
		// New event ready.
		currentEvent=event[loadfocus];
	}

	if (currentEvent)
	{
  	playnext = FALSE;
		if (playfocus>-1)
			CloseFocus(playfocus);

		playfocus=loadfocus;
		loadfocus = -1;
		user_sel = NONE;

		if (currentEvent->Type()==LOOP)
			statusLOOPING=TRUE;		/* Flag for looping */
		else
			statusLOOPING=FALSE;	/* Flag for not looping */
		
      return currentEvent->Play();
	}
	return FALSE;
}

BOOL Scheduler::LoadNextEvent(void)
{
	int loopevent=FALSE;
	CEvent *evt = GetNextEvent();
	if (evt==FALSE)
	{
		// No more events in cue (kinda benign)
		return FALSE;
	}

	statusPlaying = TRUE;
	if (evt->Type()==LOOP)
		loopevent=TRUE;

	loadfocus = evt->Open(&shandle,loopevent);
	if (loadfocus < 0)
   {
     //throw logic_error("evtOpen failed in LNE");
     DEBUGSTRING(("evtOpen failed in LNE\n"));
		return FALSE;
   }
	
	eventlist.removeabs(0);
	if (clip[loadfocus].Open(loadfocus)==FALSE)
	{
		shandle.Close(loadfocus);
		loadfocus = -1;
		DEBUGSTRING(("Error opening film.\n"));
		return FALSE; // Error opending clip
	}

	clip[loadfocus].SetupRMCallback(TriggerPlayNext,this);
   evt->LinkClip(clip[loadfocus]);
	event[loadfocus]=evt;
	return TRUE;
}
void CEvent::LinkClip(Film& clip_)
{
  clip = &clip_;
  clip->SetupTriggers(&event);
}
FILMTYPE CEvent::GetFILMTYPE() const
{
  return type;
}
FILMTYPE Scheduler::PeekAtNextEvent(void)
{
   CEvent *evt = &eventlist[0];
   if (evt)
   {
	   return evt->GetFILMTYPE();//type;
   }
   return UNKNOWN;
}

CEvent *Scheduler::GetNextEvent(void)
{
	return &eventlist[0];
}

//BOOL Scheduler::RemoveByHandle(int h) 
//{ 
//   CEvent *evt = GetEventByHandle(h);
//   if (evt)
//   {
//      delete evt;
//	  return TRUE;
//   }
//   return FALSE;
//}

//CEvent *Scheduler::GetEventByHandle(int which)
//{
//   CEvent *evt = eventlist.ptr(which);
//   if (evt)
//      eventlist.remove(which);
//
//   return evt;
//}

BOOL Scheduler::Remove(CEvent *evt)
{
#if 1
  if (eventlist.removeptr(evt)==-1)
  {
    // it's in the nodes.
	 for (int i=0; i<NODES; i++)
	 {
	 	if (evt == event[i])
	 	{
		  // Found it
		  evt->Clip()->Close();
		  return TRUE;
	 	}
	 }
	 return FALSE;
  }
#else
	if (RemoveByHandle(evt->linkhandle)==NULL)
	{
		//Item is in use
		for (int i=0; i<NODES; i++)
		{
			if (evt->linkhandle == event[i]->linkhandle)
			{
				// Found it
				evt->Clip()->Close();
				return TRUE;
			}
		}
		return FALSE;
	}
#endif
	return TRUE;
}

/********************************************************************\
\********************************************************************/

int CEvent::AddTrigger(DWORD frame,FilmCallback fn,LPVOID user)
{
   PEventStr data = DBG_NEW(EventStr);//new EventStr;
   if (data==NULL)
      return -1;

   data->frame = frame;
   data->fn=fn;
   data->user=user;
   return event.addptr(data);
}

BOOL CEvent::RemoveTrigger(int h)
{
   PEventStr data=event.ptr(h);
   if (data)
   {
      delete data;
	  event.remove(h);
	  return TRUE;
   }
   return FALSE;
}

/********************************************************************\
\********************************************************************/

BOOL Scheduler::Open(HWND h, FRSRC *rc)
{
  int err;

  WAVEFORMATEX wf={ 1,1,22050,22050,1,8,0 };
  currentEvent = NULL;

  ClearLeadinIDs();
  Empty();
  err = sound.Open(&wf,SND_SIZE<<10);							// Direct Sound
  err	+= renderer.Open(h);								// Renderer
  err += shandle.Setup(NODES,NORMAL_BUFFER_SZ,LOOP_BUFFER_SZ);	// SmartHandle
  err += ravout.Open(&renderer, &sound, 16);				// Output 16k buffer
  if (err==4) // All return TRUE (1)
  {
    event = DBG_NEW_N(PCEvent,NODES);//PCEvent[NODES];
    clip = DBG_NEW_N(Film,NODES);//new Film[NODES];
    if (event && clip)
    {
      for (int i=0; i<NODES; i++)
      {
        clip[i].Setup(&ravout,&shandle);			// Setup clips
      }
      playfocus=-1;
      outputTimer.Set(VIDEO_PERIOD,OutputChecker,this);
      shandle.SetFileReadPeriod(FILE_PERIOD);
      SetArchieveAttrib(rc);
      statusOPEN=TRUE;
      return TRUE;
    }
  }
  Close();
  return FALSE;
}

Scheduler::Scheduler()
  : bigfile(0)
  , statusPlaying(FALSE)
{
	statusPAUSED = FALSE;
	statusLOOPING = FALSE;
	statusOPEN = FALSE;
	playfocus = loadfocus = - 1;
	playnext = FALSE;

	leadin_count = 0;
	ClearLeadinIDs();

	currentEvent=NULL;
	event=NULL;
	user_sel = NONE;
	clip = NULL;
	SetupLoop();
}

Scheduler::~Scheduler()
{
	Close();
}

void Scheduler::Empty(void)
{
   /* Delete all events in list */
   eventlist.Flush();//KILL_DELETE_LINK(eventlist);

   if (currentEvent)
   {
	  currentEvent->Stop();
	  currentEvent->Close();
     delete currentEvent;
      currentEvent = NULL;
   }
   
   if (loadfocus == playfocus)
   {
	   CloseFocus(loadfocus);
	   playfocus = loadfocus;
   } else {
	   CloseFocus(loadfocus);
	   CloseFocus(playfocus);
   }

   user_sel	= NONE;
   WipeOtherLeadins();
	// ClearLeadIDs();
}

void Scheduler::Close(void)
{
    Empty();

	if (statusOPEN==FALSE)
		return;

	delete[] clip;
	clip = NULL;
	
	delete[] event;
	event = NULL;

	ravout.Close();
	renderer.Close();
	sound.Close();
	shandle.Shutdown();

	statusOPEN=FALSE;
}


/********************************************************************\
\********************************************************************/
CEvent *Scheduler::AddEntry(long entryid, FILMTYPE tp, FADETYPE effect)
{ 
   int loop = 1;
   if (tp==LOOP)
	   loop = masterLoopCount;

   CEvent *event = DBG_NEW(CEvent(NULL,tp,loop,effect));
   event->AssignID(entryid,BYENTRY);
   eventlist.addptr(event);
   return event;
}

void Scheduler::SetupLoop(int loop, ORDER_NO leadin)
{
	defaultLeadin=leadin;
	masterLoopCount=loop;
}

void Scheduler::PurgeLeadinsFromList(void)
{
   while(PeekAtNextEvent()==LEADIN)
   {
      eventlist.removeabs(0);
   }
}

void Scheduler::WipeOtherLeadins(void)
{ 
  leadin_count = 0;
  int value = (int)user_sel;
  for (int i=0; i<NODES-1; i++)
  {
    // Kill other leadins
    if (value != i)
    {
      if (leadinID[i] != -1)
      {
        shandle.Close(leadinID[i]);
      }
    }
  }
  ClearLeadinIDs();
}

void Scheduler::ForgetAll(void)
{
	Empty();
}

void Scheduler::ClearLeadinIDs()
{
  for (int i=0; i<NODES; i++)
    leadinID[i] = -1;
}

void Scheduler::CloseFocus(int &focus)
{
	if (focus<0)
		return;

	clip[focus].Stop();
	clip[focus].Close();
	shandle.Close(focus);
	focus = -1;
}

void Scheduler::CloseArchive()
{
  // deletes buffers and shuts down threads, I hope.
	shandle.Shutdown();
}

void Scheduler::OpenArchive()
{
  // allocates buffers and establishes threads.
  shandle.Setup(NODES,NORMAL_BUFFER_SZ,LOOP_BUFFER_SZ);
  shandle.SetArchieveAttrib(bigfile);
  shandle.SetFileReadPeriod(FILE_PERIOD);
  for (int i=0; i<NODES; i++)
  {
    clip[i].Setup(&ravout,&shandle);			// Setup clips
  }
  playfocus=-1;
}
